; Program 5730_sub.ASM
; 
; Example sub-routines for communicating with the 5730 via its 3 wire bus and 
; setting up its control registers.
;  
; Code is provided as is.  If this looks like a hack it is, but should provide 
; a good basis for further code development.  Read the code in conjunction
; with the data sheet.  I hope that I have commented the routines well
; enough for them to be related to the relevant parts of the data sheet.
; No attempt has been made to explore or demonstrate all the features of  
; the device.
;
; I hope this helps anyone wanting to develop PIC based 5730 devices.  
; Please write and tell me if you find it useful.
;
; Copyright Kayode Ayandokun March 2000  kayode.ayandokun@virgin.net
; This code may be duplicated in whole or in part provided that [1] there
; is no commercial gain involved in the duplication, and [2] that this
; copyright notice is preserved on all copies. 
;  


;------ Delay macro 
DELAY_SHORT MACRO                  ; provides nominal 25 usec delay
        MOVLW .5
        MOVWF LOOP2
;DELAY_SHORT_1:
	CLRWDT
        DECFSZ LOOP2, F
	GOTO	$-2 	;DELAY_SHORT_1
        ENDM

	
HIGH_SDA MACRO                  ; 
        BSF PORTA, SDA          ; 
        ENDM

LOW_SDA	MACRO
        BCF PORTA, SDA  
        ENDM

HIGH_SCL MACRO
        BSF PORTA, SCL         
        ENDM

LOW_SCL	MACRO
        BCF PORTA, SCL
        ENDM

CLOCK_PULSE MACRO	                    ; SCL momentarily to logic one
        LOW_SCL
        HIGH_SCL
        ENDM    


; ------ Routine to send 16-bit message to 5730
WRITE_WORD MACRO
        CALL 	OUT_BYTE           	; Write to I2C Bus
	MOVF	O_BYTE2, W
	MOVWF	O_BYTE
        CALL 	OUT_BYTE           	; Write to I2C Bus
 
	ENDM

; ------ Routine to send 8-bit message to 5730
WRITE_BYTE MACRO
        CALL 	OUT_BYTE           	; Write MSB to I2C Bus

	ENDM

; ----- Routine to send 0-bit message to 5730
WRITE_ZERO MACRO
	HIGH_SCL
	LOW_CSN
        HIGH_CSN

	ENDM

; ----- Lower the 5730 chip select
LOW_CSN MACRO
	BCF	PORTA, CSN
	ENDM

; ----- Raise the 5730 chip select
HIGH_CSN MACRO
	BSF	PORTA, CSN
	ENDM




; ------ Routine to set the contents of the STV5730 registers
SET_REGISTERS:

	MOVLW	INIT_5730
	MOVWF	LOOP1
RESET_LOOP:
;------ Reset bytes, see page 13
;------ 1st reset
	MOVLW	0DBH			; Set LSB of data to be transmitted... 
	MOVWF	O_BYTE2			; ... into second byte
	MOVLW	00H			; Set MSB of data to be transmitted...
	MOVWF	O_BYTE			; ... into first byte
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_WORD			; Write the word with the macro
	CALL	STOP			; Signal the end the serial bus transmission

;------ 2nd reset
	MOVLW	00H			; Set LSB of data to be transmitted... 
	MOVWF	O_BYTE2			; ... into second byte
	MOVLW	010H			; Set MSB of data to be transmitted...
	MOVWF	O_BYTE			; ... into first byte
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_WORD			; Write the word with the macro
	CALL	STOP			; Signal the end the serial bus transmission

;------ Actual reset bytes x2
	MOVLW	00H			; Set LSB of data to be transmitted... 
	MOVWF	O_BYTE2			; ... into second byte
	MOVLW	030H			; Set MSB of data to be transmitted...
	MOVWF	O_BYTE			; ... into first byte
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_WORD			; Write the word with the macro
	CALL	STOP			; Signal the end the serial bus transmission

	MOVLW	030H			; Set MSB of data to be transmitted...
	MOVWF	O_BYTE			; ... into first byte
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_WORD			; Write the word with the macro
	CALL	STOP			; Signal the end the serial bus transmission


; Second transmission
;------ 1st reset
	MOVLW	0DBH			; Set LSB of data to be transmitted... 
	MOVWF	O_BYTE2			; ... into second byte
	MOVLW	00H			; Set MSB of data to be transmitted...
	MOVWF	O_BYTE			; ... into first byte
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_WORD			; Write the word with the macro
	CALL	STOP			; Signal the end the serial bus transmission

;------ 2nd reset
	MOVLW	00H			; Set LSB of data to be transmitted... 
	MOVWF	O_BYTE2			; ... into second byte
	MOVLW	010H			; Set MSB of data to be transmitted...
	MOVWF	O_BYTE			; ... into first byte
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_WORD			; Write the word with the macro
	CALL	STOP			; Signal the end the serial bus transmission

	DECFSZ	LOOP1, F
	GOTO	RESET_LOOP

	MOVLW	INIT_5730
	MOVWF	LOOP1
REGISTER_LOOP:

	
;------ Set address pointer to register 1
	MOVLW	b'11001100'		; LSB for register address,STRU = 11, DEPL = 01100
	MOVWF	O_BYTE2			; ... into second byte
	MOVLW	00H			; MSB for register address, BUF = 00H
	MOVWF	O_BYTE			; ... into first byte
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_WORD			; Write the word with the macro
	CALL	STOP			; Signal the end the serial bus transmission

;------ Set Zoom register to initial values
	MOVLW	b'00000000'		; LSB character height and width set to 1 dot per pixel
	MOVWF	O_BYTE2			; ... into second byte
	MOVLW	b'00010000'		; MSB data message, ditto char dimensions
	MOVWF	O_BYTE			; ... into first byte
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_WORD			; Write the word with the macro
	CALL	STOP			; Signal the end the serial bus transmission

;------ Set Color register to initial values
	MOVLW	b'00000100'		; LSB char border black, char background red 
	MOVWF	O_BYTE2			; ... into second byte
	MOVLW	b'00010010'		; MSB data message, blue sceen backgrd
	MOVWF	O_BYTE			; ... into first byte
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_WORD			; Write the word with the macro
	CALL	STOP			; Signal the end the serial bus transmission

;------ Set Control register to initial values
	MOVLW	b'11010100'		; LSB bit0 = 1 for full page mode, = 0 for mixed mode, etc see pg 9
	MOVWF	O_BYTE2			; ... into second byte
	MOVLW	b'00011011'		; MSB bit0 = 1 for PAL, = 0 for NTSC, bit3 = 1 form normal full page mode
	MOVWF	O_BYTE			; ... into first byte
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_WORD			; Write the word with the macro
	CALL	STOP			; Signal the end the serial bus transmission

;------ Set Position register to initial values
	MOVLW	b'00011100'		; LSB horizontal position 28, vertical position 24, see pg 10
	MOVWF	O_BYTE2			; ... into second byte
	MOVLW	b'00010110'		; MSB "
	MOVWF	O_BYTE			; ... into first byte
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_WORD			; Write the word with the macro
	CALL	STOP			; Signal the end the serial bus transmission

;------ Set Mode register to initial values
	MOVLW	b'00011110'		; LSB see pg 10
	MOVWF	O_BYTE2			; ... into second byte
	MOVLW	b'00011000'		; MSB "
	MOVWF	O_BYTE			; ... into first byte
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_WORD			; Write the word with the macro
	CALL	STOP			; Signal the end the serial bus transmission

	CALL SET_ROWS


	DECFSZ	LOOP1, F
	GOTO	REGISTER_LOOP

	RETURN

SET_ROWS:
;------ Set row attributes 0 - 10 are the same
;------ Set address pointer to row attribute
	MOVLW	b'11000000'		; LSB for row attribute address,STRU = 11, DEPL = 00000
	MOVWF	O_BYTE2			; ... into second byte
	MOVLW	00H			; MSB for register address, BUF = 00H
	MOVWF	O_BYTE			; ... into first byte
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_WORD			; Write the word with the macro
	CALL	STOP			; Signal the end the serial bus transmission

;------ Set attributes for line 1
	MOVLW	b'01000000'		; LSB display characters with border, no offsets pg 8
	MOVWF	O_BYTE2			; ... into second byte
	MOVLW	b'00010000'		; MSB data message
	MOVWF	O_BYTE			; ... into first byte
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_WORD			; Write the word with the macro
	CALL	STOP			; Signal the end the serial bus transmission

	MOVLW	.9			; Set the line count into the counter register
	MOVWF	LINE_COUNTER
LINE_LOOP:	
	MOVLW	b'01000000'		; LSB display characters with border, no offsets pg 8
	MOVWF	O_BYTE			; ... into second byte
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_BYTE			; Write the byte with the macro
	CALL	STOP			; Signal the end the serial bus transmission

	DECFSZ	LINE_COUNTER, F
	GOTO	LINE_LOOP

;------ Set attributes for line 11, which is where we put the text
	MOVLW	b'11000111'		; LSB display characters with border, vertical offset of +7 pg 8
	MOVWF	O_BYTE2			; ... into second byte
;	MOVLW	b'00011111'		; MSB data message, horizontal offset of 15 pixels  pg 8
	MOVLW	b'00010000'		; MSB data message, horizontal offset of 0 pixels  pg 8
	MOVWF	O_BYTE			; ... into first byte
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_WORD			; Write the byte with the macro
	CALL	STOP			; Signal the end the serial bus transmission

	RETURN




;------ This routine writes space chars into all screen character locations
BLANK_LINES:

;------ Write the text into the ram buffer
;------ Set address pointer to line 1
	MOVLW	b'00000000'		; LSB for row attribute address,STRU = 00, DEPL = 00000
	MOVWF	O_BYTE2			; ... into second byte
	MOVLW	b'00000000'		; MSB for register address, BUF = line num
	MOVWF	O_BYTE			; ... into first byte
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_WORD			; Write the word with the macro
	CALL	STOP			; Signal the end the serial bus transmission


;------ Write first space character 
	MOVLW	b'00001011'		; LSB Space char 0BH, no blinking
	MOVWF	O_BYTE2			; ... into second byte
	MOVLW	b'00010111'		; MSB data message, no character background, white text
	MOVWF	O_BYTE			; ... into first byte
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_WORD			; Write the word with the macro
	CALL	STOP			; Signal the end the serial bus transmission

	MOVLW	.11
	MOVWF	LOOP1
BLANK_LINE:
	MOVLW	.28
	MOVWF	LOOP2
BLANK_CHAR:
	MOVLW	b'00001011'		; LSB Space char 0BH, no blinking
	MOVWF	O_BYTE			; 
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_BYTE			; Write the byte with the macro
	CALL	STOP			; Signal the end the serial bus transmission

	DECFSZ	LOOP2, F
	GOTO	BLANK_CHAR

	DECFSZ	LOOP1, F
	GOTO	BLANK_LINE

	RETURN



; The following routines are low level I2C routines adapted
; for the 5730.

OUT_BYTE:                       ; send o_byte on I2C bus
        MOVLW .8
        MOVWF _N
OUT_BIT:
        BCF STATUS,C            ; clear carry
        RLF O_BYTE, F           ; left shift, most sig bit is now in carry
        BTFSS STATUS, C         ; if one, send a one
        GOTO OUT_ZERO
        GOTO OUT_ONE

OUT_ZERO:
        LOW_SDA            ; SDA at zero
        CLOCK_PULSE        
        HIGH_SDA
        GOTO OUT_CONT
OUT_ONE:
        HIGH_SDA           ; SDA at logic one
        CLOCK_PULSE
        GOTO OUT_CONT
OUT_CONT:
        DECFSZ _N, F		; decrement index
        GOTO OUT_BIT
        RETURN  

;;;;;;
                
NACK:                           ; bring SDA high and clock
        HIGH_SDA
        CLOCK_PULSE
        RETURN

ACK:
        LOW_SDA
        CLOCK_PULSE
        RETURN

START:                          
	HIGH_SCL
	LOW_CSN
        LOW_SCL
        RETURN

STOP:
        HIGH_SCL
        HIGH_CSN
        RETURN





