; Program VIDEO.ASM
; 
; A simple program to write a line of characters along the bottom of a frame 
; using the 5730.  Code is provided as is.  If this looks like a hack it is,
; but should provide a good basis for further code development.  Read the code 
; in conjunction with the data sheet.  I hope that I have commented the routines
; well enough for them to be related to the relevant parts of the data sheet.
; No attempt has been made to explore or demonstrate all the features of the 
; device.
;
; I hope this helps anyone wanting to develop PIC based 5730 devices.
; Please write and tell me if you find it useful.
;
; Copyright Kayode Ayandokun March 2000  kayode.ayandokun@virgin.net
; This code may be duplicated in whole or in part provided that [1] there
; is no commercial gain involved in the duplication, and [2] that this
; copyright notice is preserved on all copies.  


          LIST p=16F84
#include <p16F84.inc>
;----- Port pin assingnments PortA ----------
; See scl sda and csn constants

          __CONFIG 3FF9H

	CONSTANT 	BASE_VAR=0CH

        CONSTANT	SCL=0
        CONSTANT	SDA=1          		; bits on PortA defined
	CONSTANT	CSN=2

	CONSTANT	INIT_5730=.5	

	CONSTANT	NUM_LINES=.11
	CONSTANT	LINE_LEN=.28
	CONSTANT	LINE_START=33H


COUNTER   	EQU BASE_VAR+0
LOOP1		EQU BASE_VAR+1
LOOP2		EQU BASE_VAR+2
DELAY1		EQU BASE_VAR+3
O_BYTE2  	EQU BASE_VAR+4      	; LSB to be output to 8574
O_BYTE  	EQU BASE_VAR+5    	; MSB to be output to 8574
_N      	EQU BASE_VAR+6     	; index
CHAR_COUNTER	EQU BASE_VAR+7
LINE_COUNTER	EQU BASE_VAR+8


	ORG 000H

	GOTO MAIN

	ORG 004H

#include "5730_sub.asm"

MAIN:
	CLRF PORTA
	CLRF PORTB
	CLRWDT
;---------------------------------------------------------
	BSF STATUS, RP0		; bank 1
	MOVLW	b'00010110'	; Setting for: PORTB pull ups enabled, INTF on falling edge, the prescaler assigned to TMR0, 1:128 ratio of the precscaler
	MOVWF	OPTION_REG


	MOVLW 00H		; Input / Output setting for port B 
	MOVWF TRISB

	MOVLW 00H		; All outputs 
	MOVWF TRISA

	BCF STATUS, RP0     	; back to bank 0
;------------------------------------------------------------------

	HIGH_CSN	; Set the initial state of the 3-wire bus
	HIGH_SDA
	HIGH_SCL

;------ Initialise the line buffer with character codes 0 - 27
	MOVLW	LINE_START
	MOVWF	FSR
	CLRF	CHAR_COUNTER	
CHAR_LOOP1:
	MOVF	CHAR_COUNTER, W
	MOVWF	INDF
	INCF	FSR, F
	INCF	CHAR_COUNTER, F
	MOVLW	LINE_LEN
	SUBWF	CHAR_COUNTER, W
	BTFSS	STATUS, Z
	GOTO	CHAR_LOOP1

	CLRF	LINE_COUNTER
;------------------------------------------------

	MOVLW	.1			; Set 250 ms delay, aim is for the 5730 to reach equilbrium
	CALL	DELAY_LOOP

	CALL 	SET_REGISTERS		; Set up the STV5730A registers
	CALL	BLANK_LINES		; Set all characters on all lines to be blank
	CALL 	WRITE_CHARS		; Write the characters in the character buffer to the 5730

;------------------------------------------------


MAINLOOP:

	CALL	EACH_SECOND				; Update the display buffer once each second

	CALL 	WRITE_CHARS				; Write the characters in the buffer to the 5730

	MOVLW	.4					; Set one second delay
	CALL	DELAY_LOOP

	GOTO MAINLOOP


; ----- Routine to write characters from memory buffer to the 5730 
WRITE_CHARS:
	CALL	SET_ROWS		; Set all characters on all lines to be blank

;------ Write the text into the ram buffer
;------ Set address pointer to line 11 
	MOVLW	b'00000001'		; LSB for row attribute address,STRU = 00, DEPL = 00000
	MOVWF	O_BYTE2			; ... into second byte
	MOVLW	b'00001010'		; MSB for register address, BUF = .10 points to 11th line
	MOVWF	O_BYTE			; ... into first byte
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_WORD			; Write the word with the macro
	CALL	STOP			; Signal the end the serial bus transmission

;------ Successively write the characters from the line buffer to the 5730
	MOVLW	LINE_START		; Set the FSR register to point to the start of the line
	MOVWF	FSR
	MOVLW	LINE_LEN		; Set the line count into the character counter register
	MOVWF	CHAR_COUNTER

;------ Write first character as a 16 bit word. LSB is character code, MSB controls character attributes.  See pg 20.  
	MOVF	INDF, W			; load the character into the output variable, LSB of word
	MOVWF	O_BYTE2			; bit 7 controls blinking attrib. no blinking = 0, blinking = 1
	MOVLW	b'00011111'		; MSB data message, no character background = bit3 0, character background = bit3 1, white text
	MOVWF	O_BYTE			; ... into first byte
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_WORD			; Write the word with the macro
	CALL	STOP			; Signal the end the serial bus transmission

	DECF	CHAR_COUNTER, F
	INCF	FSR, F
	
;------ Now write the rest of the line 
CHAR_LOOP:
	MOVF	INDF, W			; LSB load the character into the output variable
	MOVWF	O_BYTE			; 
        CALL 	START			; Signal the start the serial bus transmission
	WRITE_BYTE			; Write the byte with the macro
	CALL	STOP			; Signal the end the serial bus transmission

	INCF	FSR, F			; Move the pointer to the next char
	DECFSZ	CHAR_COUNTER, F
	GOTO	CHAR_LOOP

	RETURN




;------ Update the display buffer
EACH_SECOND:

;------ Increment the character codes in the line buffer
	MOVLW	LINE_START
	MOVWF	FSR
	MOVLW	LINE_LEN
	MOVWF	CHAR_COUNTER
CHAR_LOOP3:
	INCF	INDF, F
	INCF	FSR, F
	DECFSZ	CHAR_COUNTER, F
	GOTO	CHAR_LOOP3

	RETURN


DELAY_LOOP:
	MOVWF	DELAY1
LOOP1D:
	CALL	DELAY_250

	DECFSZ	DELAY1, F
	GOTO	LOOP1D

	RETURN 




DELAY_250:                     ; provide 250 ms delay
        MOVLW .250
        MOVWF LOOP1
OUTER:
        MOVLW   .110            ; close to 1.0 msec delay when set to .110
        MOVWF   LOOP2
INNER:
        NOP
        NOP
        NOP
        NOP
        NOP
        NOP
        DECFSZ  LOOP2, F        ; decrement and leave result in LOOP2 
                                ; skip next statement if zero
        GOTO INNER
        DECFSZ  LOOP1, F
        GOTO OUTER
        RETURN

	END
